#include "bits.nt"

;
; title:   trosy-fb_h2oex2df.ncf
; summary: H2O-exchange utility program
; author:  Nick Fitkzee (nfitzkee@nih.gov)
; date:    July 2, 2010
;
; pulse lengths and amplifier configuration (DRX, 800 MHz):
;
; * Proton Channel (H=f1, carrier on H2O)
;   p1    =  90 deg (10us)   pulse @ pl1
;   p9    = 180 deg (2ms)    pulse @ sp10 (Sinc1.1000)
;   p2    =  90 deg (1.2ms)  pulse @ sp9  (Sinc1.1000, phcor21)
;         =  90 deg (1.2ms)  pulse @ sp0  (Sinc1.1000, phcor11)
;         =  90 deg (1.2ms)  pulse @ pl12 (phcor10)
;   p25   =  90 deg (1ms)    pulse @ sp1  (Sinc1.1000, phcor22=phcor23)
;         =  90 deg (1ms)    pulse @ sp2  (Sinc1.1000, phcor20)
;   p29   =  ~1 deg (1us)    pulse @ pl20 (spy pulse) 
;
; * Nitrogen Channel (N=f2, carrier on ~117.5 ppm, in0 = 0.5/sw, loop l3)
;   p7    = 90 deg (50us)    pulse @ pl7
;
; * Carbon Channel (C1=f4 carrier on 56ppm, C2=f5 carrier on 176ppm)
;   p4    = 180 deg (17.8us) pulse @ pl4 (C1) or pl6 (C2)
;   pcpd5 = garp decoupling (1.5 kHz field) @ pl31
;
; For Avance systems or system with one 13C amplifier, you must
; modify the pulse program so that the carrier is on 176 ppm and 
; the Ca decoupling pulse uses a shaped pulse with an offset.
;
; * Gradient pulses
;   p20   =     2m  -> gp10
;   p21   =     1m  -> gp2, gp3
;   p22   = 101.3u  -> gp4
;   p23   =   500u  -> gp5, gp6
;
;   gp0   (  5% z)  (gron)      : On during HX mixing time
;   gp1   (  1% z)  (gron)      : Suppress radiation damping during INEPT
;   gp2   (-10% z)  (sine.50)   : S3E refocussing
;   gp3   ( 30% z)  (sine.50)   : S3E refocussing
;   gp4   ( 60% z)  (sine.20)   : Decoding gradient
;   gp5   (-60% z)  (sine.20)   : 1/2 encoding gradient
;   gp6   ( 60% z)  (sine.20)   : 1/2 encoding gradient
;   gp10  (0.5% z)  (sine.100)  : ZZ crusher
;
; The vdlist controls the length of the mixing times (with l6 total mixing
; times).  Each entry in vdlist is multiplied by two, so if vdlist specifies
; 10 ms, the actual time will be 20 ms.

; delays and phasing

"d11=50m"
"d21=2.4m-10u"
"d22=2.3m-p21-p25"
"d23=2.3m-p21"
"d26=d23+p7-p2-p1*0.818+p22*0.5+de*0.5+5u"
"d27=d23+p7-p2-p1*1.182-p22*0.5-de*0.5-15u"

; Uncomment for 15N 0,0 phasing
;"d25=200u"
;"d0=d25*0.5+p7*0.637-p4*2-p1*0.5-2u"

; Uncomment for 15N -90,180 phasing
"d25=200u"
"d0=in0*0.5+d25*0.5+p7*0.637-p4*2-p1*0.5-2u"

;- Pulse Program Control ----------------------------------------------------

; When using spy pulses, TD = l8 = ~8, ns = 1, d1 ~ 2.0 s.  Read the water
; peak from the final FID.

; To measure W0/ W+, uncomment the following lines:
;#define SPY_PULSE
;#define SPY_REFERENCE

; To measure W-, uncomment the following lines:
;#define SPY_PULSE
;#define SPY_INVERSION


; For the reference experiment, TD = l3 * 2, ns >= 8, d1 >= 8.0 s.
; To measure the reference experiment (I0/M0), uncomment the following line:
;#define REFERENCE

; Leave all of the above commented to measure the actual HX measurement.
; For the HX experiment, TD = l3*l6*4, ns >= 8, d1 ~ 2.0.

;- End Pulse Program Control ------------------------------------------------

"p29=1u"
"d9=p9"

#ifdef REFERENCE
#define PLUS_Z
#endif

#ifdef PLUS_Z
"l5=2"
#else
"l5=1"
#endif

"l7=1"

1       ze
        1m RESET
2       100u do:C2 do:N
        900u LOCK_ON        
        d11
3       1m 
4       1m 
5       1m
6       1m
        d1 

        1m LOCK_OFF
	10u pl1:H
        10u pl7:N
        10u pl6:C2
        10u pl4:C1

#ifdef SPY_REFERENCE
        if "l7==l8" goto 770
#endif

#ifndef REFERENCE

;-- Water flip phase: +/- z -------------------------------

20	3u pl0:H
        3u 
        if "l5==1" goto 30
        d9               ; No flipback applied   
        goto 40
30     (p9:sp10 ph6):H   ; Flip to -z
40	3u
        3u pl4:C1 pl1:H
;goto 777

#ifdef SPY_INVERSION
#ifndef PLUS_Z
        if "l7!=l8" goto 45
       (p1*2 ph0):H
        3u
#endif  ;-- PLUS_Z
#endif  ;-- SPY_INVERSION

#ifdef SPY_PULSE
45      10u gron0         ; eliminate radiation damping
        10m
        500u groff
#else
45      5u gron0
        vd                ; must not exceed 400 ms
        5u groff
        10u
        5u gron0
        vd                ; must not exceed 400 ms
        5u groff
        500u
#endif
#ifdef SPY_INVERSION
        if "l7==l8" goto 770
#endif  ;-- SPY_INVERSION

#endif  ;-- ifndef REFERENCE

;-- Start INEPT to 15N ------------------------------------

60      5u pl1:H
        5u
       (p1 ph0):H
        5u gron1
        d21
        5u groff  
       (center (p1*2 ph0):H (p7*2 ph0):N)
        5u gron1
        d21
        5u groff
       (p1 ph1):H
        3u pl0:H
        3u
        if "l5==1" goto 70
       (p2:sp9 ph21:r):H      ; Water starts +z: std flip
        goto 80
70     (p2:sp0 ph11:r):H      ; Water starts -z: inv flip
80      3u
        3u pl1:H

;goto 777
        p20:gp10
85      200u

;-- TROSY Evolution and back transfer ---------------------

        if "l1==1" goto 88
       (p7 ph17):N
        105u
        p23:gp6
        d25
       (p7*2 ph17):N 
        p23:gp5
        goto 89

88     (p7 ph7):N
        105u
        p23:gp5
        d25
       (p7*2 ph7):N 
        p23:gp6

89      105u
        d0 
       (p4*2 ph0):C1
        4u
       (p4*2 ph0):C2
        d0

        if "l1==1" goto 78
       (p1 ph2):H
        3u
        2u pl2:H
       (p25:sp1 ph22:r):H
        goto 79

78     (p1 ph12):H
        3u
        2u pl0:H
       (p25:sp1 ph23:r):H

79      3u
        2u pl1:H
;goto 777
        p21:gp2
        d22
        300u
       (center (p1*2 ph0):H (p7*2 ph0):N)
        p21:gp2
        d22
        305u pl0:H
	(p25:sp2 ph20:r):H
        3u
        2u pl1:H
        if "l1==1" goto 98
       (p1 ph0):H (p7 ph4):N     ;DOUBLE 90
        goto 99
98     (p1 ph0):H (p7 ph14):N    ;DOUBLE 90
99      5u
;goto 777
        p21:gp3           
        295u pl12:H
       (d26 p2 ph10:r 3u 2u pl1 p1*2 ph0 3u 2u pl12 p2 ph10:r d27):H (d23 p7*2 ph0 d23):N
        5u pl1:H
;goto 777
        p21:gp3
        295u pl31:C2
       (p7 ph0):N

#ifdef SPY_FINAL
        if "l7==l8" goto 770
#endif

        goto 776
770     3u
        2u pl20:H
       (p29 ph30):H
        goto 777

776     p22:gp4 
777     10u pl31:C2
        go=2 ph31 cpds5:C2
        100u do:N do:C2
        100u LOCK_ON
        d11 wr #0 if #0 zd

#ifdef SPY_PULSE
        800u iu7
        lo to 3 times l8
#else
        800u iu5
#ifndef REFERENCE
        lo to 3 times 2
#endif
#endif

        500u ru5
        500u ivd

#ifndef SPY_PULSE
#ifndef REFERENCE
        lo to 4 times l6
#endif
#endif

        100u iu1
        900u

#ifndef SPY_PULSE
        lo to 6 times 2
#endif

        100u id0
        100u ru1
        100u ip31*2     ; generally needed to move streak at 15N
        100u ip7*2      ; carrier to edge of spectrum, since d1
        100u ip17*2     ; is less than H2O T1.
        500u

#ifndef SPY_PULSE
        lo to 6 times l3
#endif SPY_PULSE

        1m rd0
1m do:C2 do:N
1m LOCK_ON
1m RESET
1m
exit    
        
ph0=0
ph1=1 1 1 1 3 3 3 3     ;original setting1     
                        ;check right phase for Boltzmann !!!!!
ph30=0 2 2 0 1 3 3 1
ph2=3                   ;original setting
ph4=3                   ;original setiing
ph5=1

ph6=0
ph16=2

ph15=1
ph10=2
ph20=0
ph22=1
ph23=3
ph11=2 ;3 3 3 3 1 1 1 1
ph21=0 ;

ph12=1                  ;original seting
ph7=1 0 3 2  ;3 2 1 0
ph17=1 2 3 0 ;3 0 1 2
ph14=1                  ;original setting

#ifdef SPY_PULSE
ph31=0 2 2 0 1 3 3 1
#else
ph31=1 2 3 0 3 0 1 2
#endif
