
RES_3TO1 = {'ALA': 'A', 'CYS': 'C', 'ASP': 'D', 'GLU': 'E', 'PHE': 'F',
            'GLY': 'G', 'HIS': 'H', 'ILE': 'I', 'LYS': 'K', 'LEU': 'L',
            'MET': 'M', 'ASN': 'N', 'PRO': 'P', 'GLN': 'Q', 'ARG': 'R',
            'SER': 'S', 'THR': 'T', 'VAL': 'V', 'TRP': 'W', 'TYR': 'Y'}

RES_1TO3 = {'A': 'ALA', 'C': 'CYS', 'D': 'ASP', 'E': 'GLU', 'F': 'PHE',
            'G': 'GLY', 'H': 'HIS', 'I': 'ILE', 'K': 'LYS', 'L': 'LEU',
            'M': 'MET', 'N': 'ASN', 'P': 'PRO', 'Q': 'GLN', 'R': 'ARG',
            'S': 'SER', 'T': 'THR', 'V': 'VAL', 'W': 'TRP', 'Y': 'TYR'}
            

import re

pat = re.compile('[a-zA-Z\*]+'
                 '[12345677890]+'
                 '[a-z]*'
                 '[A-Z]')

def read_fasta(fn):
    """read a fasta file, possibly containing multiple sequences"""
    
    f = open(fn)
    l = f.readline()
    s = []

    curseq = ''
    curdsc = None

    while l:
        l = l.strip()

        if l and l[0] <> '#':
            if l[0] == '>':
                if curdsc is None:
                    curdsc = l[1:]
                else:
                    s.append((curdsc, curseq))
                    curdsc = l[1:]
                    curseq = ''

            else:
                seq = ''.join(l.split())
                curseq = curseq + seq

        l = f.readline()

    if curseq:
        s.append((curdsc, curseq))
        
    f.close()
    return s

def parse_sparky(s):
    """parse a sparky assignment label with (x) = -x"""
    reslist = s.split('-')
    result = [parse_res_numeric(reslist[0])]

    for i in xrange(1, len(reslist)):
        s = reslist[i]

        if s[0] == '?' or (len(s) > 1 and (numeric(s[1]) or s[1] == '(')):
            result.append(parse_res_numeric(s))
        else:
            result.append((result[-1][0], result[-1][1], s))

    return result

def parse_res_numeric(s):
    if s[0] == '?':
        if len(s) == 1: return None, None, None
        return None, None, s[1:]
    
    rtype = s[0]
    rppos = s.find(')')

    if rppos >= 1:
        rnum = -int(s[2:rppos])
        atype = s[rppos+1:]
    else:
        for i in xrange(1, len(s)):
            if not numeric(s[i]): break
        rnum = int(s[1:i])
        atype = s[i:]

    return rnum, rtype, atype

class Resonance:
    def __init__(self, idx=None, rtype=None, atom=None, alt=None):
        self.index = idx
        self.rtype = rtype
        self.alt   = alt
        self.atom  = atom

    def isUnknown(self):
        return self.index == None or self.rtype == None or self.atom == None

    def toString(self, prev=None):
        if prev != None:
            if prev.index == None or prev.rtype == None or \
                   prev.atom == None:
                return '?'
            if prev.index == self.index and prev.rtype == self.rtype and \
                   prev.alt == self.alt and self.atom != None:
                return self.atom.upper()

        if self.index == None or self.rtype == None or self.atom == None:
            return '?'

        istr = '%i' % self.index
        if self.index < 0: istr = '(%i)' % abs(self.index)

        if self.alt == None:
            altstr = ''
        else:
            altstr = self.alt.lower()

        
        return '%s%s%s%s' % (self.rtype, istr, altstr, self.atom.upper())

    def _complete(self, s):
        nums = '0123456789'
        return not (pat.match(s) == None)

    def __str__(self):
        return self.toString()

    def __eq__(a, b):
        try:
            return a.index == b.index and a.rtype == b.rtype and \
                   a.alt == b.alt and a.atom == b.atom
        except:
            return False

    def __hash__(a):
        return a.__str__().__hash__()

    def __gt__(a, b):
        if a.alt != None and b.alt != None:
            if a.index == b.index:
                return a.alt > b.alt
            return a.index > b.index
        return a.index > b.index

    def __ge__(a, b):
        if a.alt != None and b.alt != None:
            if a.index == b.index:
                return a.alt >= b.alt
            return a.index >= b.index
        return a.index >= b.index

    def __le__(a, b):
        if a.alt != None and b.alt != None:
            if a.index == b.index:
                return a.alt <= b.alt
            return a.index <= b.index
        return a.index <= b.index

    def __lt__(a, b):
        if a.alt != None and b.alt != None:
            if a.index == b.index:
                return a.alt < b.alt
            return a.index < b.index
        return a.index < b.index
    
    def fromString(self, s, prev=None):
        class IncompleteError(Exception):
            pass

        if s == '?':
            self.rtype = None
            self.alt = None
            self.index = None
            self.atom = None

            return self
        
        if not self._complete(s):
            if prev == None:
                raise IncompleteError, 'No previous string given'
            self.rtype = prev.rtype
            self.index = prev.index

            
            if s[0].lower() == s[0]:
                alt_list = []
                i = 0
                while i < len(s):
                    if s[i].lower() == s[i]:
                        alt_list.append(s[i])
                    i = i+1
                                    
                self.alt = ''.join(alt_list)
                self.atom = s[i-1:]

            else:
                self.alt = prev.alt
                self.atom = s

            return self

        rtype_list = []
        index_list = []
        alt_list = []

        i = 0


        while i < len(s):
            if s[i] in '01234567890(':
                break
            rtype_list.append(s[i])
            i = i+1
                
        if s[i] == '(':
            index_list.append('-')
            i = i+1

        while i < len(s):
            if s[i] in '01234567890':
                index_list.append(s[i])
                i = i+1
            else:
                break
            
        if s[i] == ')':
            i = i+1

        if s[i].lower() == s[i]:
            while i < len(s):
                if s[i].lower() == s[i]:
                    alt_list.append(s[i])
                    i = i+1
                else:
                    break
        try:
            self.atom = s[i:].upper()
            self.rtype = ''.join(rtype_list)
            self.index = int(''.join(index_list))
            self.alt = ''.join(alt_list)
            if not len(self.alt): self.alt = None
        except:
            raise IncompleteError, 'Unable to parse sparky format'
        
        return self
            
def parse_sparky2(s, sep="-"):
    toks = s.split(sep)
    prev = None
    result = []
    
    for t in toks:
        result.append(Resonance().fromString(t, prev))
        prev = result[-1]

    return result
    
                   
def numeric(n):
    try:
        float(n)
        return 1
    except:
        pass

    return 0
