
import string, sys

class GDBTab:
    def __init__(self, fn, quote='"'):
        self.quote = quote
        self.file = fn
        self.entries = []

        self.__read__()

    def __read__(self):
        f = open(self.file)
        l = f.readline()
        i = 0

        data = []
        self.vars = []
        self.fmts = []
        self.data = {}

        while l:
            i = i+1
            l = l.strip()

            if l and l[0] <> '#':
                try:
                    if l[0:6].upper() == 'REMARK' or \
                           l[0:4].upper() == 'NULL':
                        l = f.readline()
                        continue

                    if l[0:4].upper() == 'DATA':
                        toks = l[4:].split()
                        key = toks[0]
                        if self.data.has_key(key):
                            self.data[key] = self.data[key] + toks[1:]
                        else:
                            self.data[key] = toks[1:]
                        l = f.readline()
                        continue
                    
                    if l[0:4].upper() == 'VARS':
                        self.vars = l.split()[1:]
                        
                    elif l[0:6].upper() == 'FORMAT':
                        self.fmts = l.split()[1:]

                    else:
                        data.append(l)

                except:
                    raise
                    raise 'GDBError', 'Unable to process line %i: %s' % \
                          (i, l)
                
            l = f.readline()
        f.close()

        if len(self.vars) <> len(self.fmts):
            raise 'GDBError', 'VARS and FORMAT lines do not match in length.'

        dbstruct = {}
        
        for i in xrange(len(self.vars)):
            if self.fmts[i].find('i') >= 0 or \
               self.fmts[i].find('d') >= 0:
                func = lambda x: int(x)
                dbstruct[self.vars[i]] = self.fmts[i], func

            elif self.fmts[i].find('f') >= 0 or \
               self.fmts[i].find('e') >= 0:
                func = lambda x: float(x)
                dbstruct[self.vars[i]] = self.fmts[i], func

            elif self.fmts[i].find('s') >= 0:
                func = lambda x: x
                dbstruct[self.vars[i]] = self.fmts[i], func

            else:
                raise 'GDBError', 'Unrecognized format string %s' % \
                      self.fmts[i]

        self.dbstruct = dbstruct

        for d in data:
            toks = self.__tokenize__(d, qs=self.quote)

            if len(toks) <> len(self.vars):
                print toks
                print self.vars
                raise 'GDBError', 'Inconsistent number of variables: %s' %\
                      d

            entry = {}

            for i in xrange(len(toks)):
                fmt, func = dbstruct[self.vars[i]]
                try:
                    entry[self.vars[i]] = func(toks[i])
                except:
                    raise 'GDBError', 'Parse error: %s' % tok

            self.entries.append(entry)            

    def __write_header__(self, out=sys.stdout):
        w = out.write

        w('VARS ')
        for v in self.vars:
            w('%s ' % v)
        w('\n')

        w('FORMAT ')
        for f in self.fmts:
            w('%s ' % f)
        w('\n\n')

    def __write_dbline__(self, line, out=sys.stdout):
        w = out.write

        for i in xrange(len(self.vars)):
            col = self.vars[i]
            fmt = self.fmts[i]
            w(('%s ' % fmt) % line[col])
        w('\n')

    def __write_dbindex__(self, idx, out=sys.stdout):
        w = out.write

        for i in xrange(len(self.vars)):
            col = self.vars[i]
            fmt = self.fmts[i]
            w(('%s ' % fmt) % self.entries[idx][col])
        w('\n')
        
    def __tokenize__(self, l, qs):
        return self.__get_next_token__(l, qs, [])

    def __get_next_token__(self, l, qs, tokens):
        if not len(l): return tokens

        oquote = 0
        start = 0
        i = 0
        ws = string.whitespace
        started = 0

        while 1:
            if i == len(l):
                if oquote:
                    raise 'GDBError', 'No closing quote found: %s' % l[start:]
                end = i
                break

            if l[i] in ws:
                if started:
                    end = i
                    break
                else:
                    start = start + 1
                    
            elif l[i] in qs:
                oquote = 1
                start = start + 1
                ws = qs
                started = 1

            else:
                started = 1

            i = i+1

        if start <> end: tokens.append(l[start:end])
        return self.__get_next_token__(l[end+1:], qs, tokens)
    
    def __getitem__(self, idx):
        return self.entries[idx]

    def __len__(self):
        return len(self.entries)

class PeakTable(GDBTab):
    def __init__(self, fn, quote='"', SparkyFormat=0):
        self.quote = quote
        self.file = fn
        self.entries = []
        self.sparky = SparkyFormat

        self.__read__()
        self.__make_assignment_dict__()

    def __make_assignment_dict__(self):
        entries = self.entries
        assigns = {}
        
        for e in entries:
            key = e['ASS']
            if assigns.has_key(key):
                assigns[key].append(e)
            else:
                assigns[key] = [e]

        self.assigns = assigns

    def getAssignment(self, an):
        return self.assigns.get(an, [])

    def getResidues(self):
        entries = self.entries
        
        residues = []
        data = {}
        
        if self.sparky:
            for e in entries:
                key = e['ASS']

                if key.upper() == 'NONE' or key.find('?') >= 0:
                    continue

                cors = self.__parse_sparky_assign__(key)
                res, num, atm = cors[0]

                if data.has_key(num):
                    data[num].append(e)
                else:
                    residues.append(num)
                    data[num] = [e]

            residues.sort()
            return residues, data

        for e in entries:
            key = e['ASS']
            
            if key.upper() == 'NONE' or key.find('?') >= 0:
                continue

            try:
                num, res = int(key[:-1]), key[-1]
            except:
                try:
                    res, num = key[0], int(key[1:])
                except:
                    try:
                        num = int(key)
                    except:
                        raise 'PeakTableError: Unable to parse residue' \
                              ' name: %s' % key
            
            if data.has_key(num):
                data[num].append(e)
            else:
                residues.append(num)
                data[num] = [e]
                          
        residues.sort()
        return residues, data

    def __parse_sparky_assign__(self, assign):
        result = []
        cors = assign.split('-')
        nums = string.digits
        
        for cidx in xrange(len(cors)):
            cname = cors[cidx]

            if cidx == 0 or (len(cname) > 2 and \
                             (cname[1] in nums or cname[1] == '(')):
                res = cname[0]
            
                if cname[1] == '(':
                    close = cname.find(')')
                    num = -int(cname[2:close])
                    atm = cname[close+1:]
                else:
                    end = 1
                    for i in range(1, len(cname)):
                        if cname[i] not in nums:
                            end = i
                            break

                    num = int(cname[1:end])
                    atm = cname[end:]

            else:
                res = result[-1][0]
                num = result[-1][1]
                atm = cname

            result.append( (res, num, atm) )

        return result
