#!/bin/csh

#
# In the lines below, tauValues are the HX mixing times
# specified in the experiments (in increasing order).
# Since it's a good idea to mix up the actual order in
# the experiment, idxExpmt allows you to specify which
# plane corresponds to which mixing time.
#

set tauValues = ( 20  60 100 180  240  360  480 )
set idxExpmt  = (  1   5   3   7    6    4    2 )

rm -rf ft ucsf
mkdir ft ucsf

foreach n ( `seq 0 ${#tauValues}` )
  @ i = $n + 1
  set outName  = (`printf ft/test%03d.ft2 $i`)
  set ucsfName = (`printf ucsf/test%03d.ucsf $i`)

  if ($n == 0) then
    echo Reference Experiment...
    set inName   = (`printf ../ref/test.fid`)  # get the ref experiment
  else
    echo Time t${n} \($tauValues[$n] ms\)...
    set inName   = (`printf fid/test%03d.fid $idxExpmt[$n]`)
  endif 

  echo "* input  = $inName"
  echo "* output = $outName"

# 
# Adjust the NMRPipe commands below just like you would any 
# other processing script;
#

  nmrPipe -in $inName \
  | nmrPipe  -fn POLY -time                             \
  | nmrPipe  -fn SP -off 0.5 -end 0.98 -pow 2 -c 1.0    \
  | nmrPipe  -fn ZF -size 2048                          \
  | nmrPipe  -fn FT                                     \
  | nmrPipe  -fn EXT -x1 11ppm -xn 6ppm -sw -verb       \
  | nmrPipe  -fn PS -p0 160.4 -p1 -19.0 -di             \
  | nmrPipe  -fn TP                                     \
  | nmrPipe  -fn SP -off 0.5 -end 0.98 -pow 1 -c 1.0    \
  | nmrPipe  -fn ZF -size 1024                          \
  | nmrPipe  -fn FT                                     \
  | nmrPipe  -fn PS -p0 180.0 -p1 180.0 -di             \
  | nmrPipe  -fn TP                                     \
  | nmrPipe  -fn POLY -auto -ord 2                      \
     -verb -ov -out $outName
  
  if ($n == 0) then
    sethdr $outName -tau 0
  else
    sethdr $outName -tau $tauValues[$n]
  endif

# Optional: if you want to convert to sparky, uncomment
# the line below.
#  pipe2ucsf $outName $ucsfName

end

